==> 1.0.9-dev.tar.gz: lib/src/navigation/app_route.dart <==
import 'dart:async';

import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutterx_application/src/application/application.dart';
import 'package:flutterx_application/src/ui/fullscreen_draggable_bottom_sheet.dart';
import 'package:flutterx_application/src/ui/system_ui_style.dart';
import 'package:flutterx_application/src/utils/code.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

part 'route_activity.dart';
part 'route_bottom_sheet.dart';
part 'route_dialog.dart';

///Arguments that can be passed to a route
typedef RouteArgs = Map<String, dynamic>;

/// Builder of a route
typedef RouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args);

/// Transform result when calling [AppRoute.openForward]
typedef ResultTransform<T, R> = FutureOr<R> Function(T value);

/// Build root theme of a route before [SystemUiOverlayStyle] is applied to the route [AnnotatedRegion]
typedef ThemeBuilder = ThemeData Function(BuildContext context, RouteSettings settings);

/// Handle route accessibility
class RouteAccessibility {
  /// Check if route is accessible from current [settings]
  final bool Function(RouteSettings settings) isAccessible;

  /// [RouteSettings] to be redirected to in case [isAccessible] returns false
  /// [request] is the route that could not be accessed
  final RouteSettings Function(RouteSettings request) redirect;

  const RouteAccessibility({required this.isAccessible, required this.redirect});
}

/// A route to a navigation UI component.
///
/// Supported routes are:
/// - [ActivityRoute]
/// - [DialogRoute]
/// - [BottomSheetRoute]
///
/// [onGenerateRoute] should be passed to [MaterialApp] onGenerateRoute parameter
/// [name] of the route
/// [open] opens this route on top of the current
/// [openReplace] replaces the current route with this one and return result for the old one
/// [openForward] replaces the current route with this one and forward result to the new one.
///               This means that old route will return the result of the new route after it's returned
///               (and maybe transformed)
/// [T] the result type for this route
abstract class AppRoute<T> {
  static final Map<String, ActivityRoute> _routes = {};

  static Iterable<ActivityRoute> get routes => _routes.values;

  /// Name of route
  final String name;

  /// Accessibility rules or null for public routes
  final RouteAccessibility? accessibility;

  const AppRoute(this.name, {this.accessibility});

  RouteSettings settings({RouteArgs args = const {}}) => RouteSettings(name: name, arguments: args);

  Future<T> open(BuildContext context, {RouteArgs args = const {}});

  Future<T> openReplace(BuildContext context, {RouteArgs args = const {}, Object? result}) {
    Navigator.of(context).pop(result);
    return open(context, args: args);
  }

  Future<T> openForward<R>(BuildContext context, {RouteArgs args = const {}, ResultTransform<T, R>? transform}) {
    transform ??= (value) => value as R;
    final completer = Completer<R>();
    Navigator.of(context).pop(_ForwardResult<R>(completer.future));
    final raw = open(context, args: args);
    raw.then(transform).then(completer.complete, onError: completer.completeError);
    return raw;
  }

  static void declareRoutes(Set<ActivityRoute> routes) {
    for (final route in routes) {
      if (_routes.containsKey(route.name)) throw StateError('Route ${route.name} already defined');
      _routes[route.name] = route;
    }
  }

  static Route onGenerateRoute(RouteSettings settings) {
    if (settings.arguments == null) settings = settings.copyWith(arguments: const <String, dynamic>{});
    final route = requireNotNull(_routes[settings.name], message: () => 'Route ${settings.name} not found');
    final accessibility = route.accessibility;
    if (accessibility != null && !accessibility.isAccessible(settings)) {
      final redirect = accessibility.redirect(settings);
      return onGenerateRoute(redirect);
    }
    return route.builder(settings);
  }

  static void redirect(BuildContext context, RouteSettings? settings) => settings == null
      ? Navigator.of(context).maybePop()
      : Navigator.of(context).pushReplacementNamed(settings.name!, arguments: settings.arguments);

  static Future<T> _result<T>(Future result) async =>
      result.then((value) async => value is _ForwardResult ? await value.value : value as T);

  static String _routeName(Object data) => data is String ? data : data.toString().snakeCase;
}

class _ForwardResult<R> {
  final Future<R> value;

  const _ForwardResult(this.value);
}

==> 1.1.4-dev.tar.gz: lib/src/navigation/app_route.dart <==
import 'dart:async';

import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutterx_application/src/application/application.dart';
import 'package:flutterx_application/src/ui/fullscreen_draggable_bottom_sheet.dart';
import 'package:flutterx_application/src/ui/system_ui_style.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

part 'route_activity.dart';
part 'route_bottom_sheet.dart';
part 'route_dialog.dart';

///Arguments that can be passed to a route
typedef RouteArgs = Map<String, dynamic>;

/// Builder of a route
typedef RouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args);

/// Transform result when calling [AppRoute.openForward]
typedef ResultTransform<T, R> = FutureOr<R> Function(T value);

/// Build root theme of a route before [SystemUiOverlayStyle] is applied to the route [AnnotatedRegion]
typedef ThemeBuilder = ThemeData Function(BuildContext context, RouteSettings settings);

/// Handle route accessibility
class RouteAccessibility {
  /// Check if route is accessible from current [settings]
  final bool Function(RouteSettings settings) isAccessible;

  /// [RouteSettings] to be redirected to in case [isAccessible] returns false
  /// [request] is the route that could not be accessed
  final RouteSettings Function(RouteSettings request) redirect;

  const RouteAccessibility({required this.isAccessible, required this.redirect});
}

/// A route to a navigation UI component.
///
/// Supported routes are:
/// - [ActivityRoute]
/// - [DialogRoute]
/// - [BottomSheetRoute]
///
/// [onGenerateRoute] should be passed to [MaterialApp] onGenerateRoute parameter
/// [name] of the route
/// [open] opens this route on top of the current
/// [openReplace] replaces the current route with this one and return result for the old one
/// [openForward] replaces the current route with this one and forward result to the new one.
///               This means that old route will return the result of the new route after it's returned
///               (and maybe transformed)
/// [T] the result type for this route
abstract class AppRoute<T> {
  static final Map<String, ActivityRoute> _routes = {};

  static Iterable<ActivityRoute> get routes => _routes.values;

  /// Name of route
  final String name;

  /// Accessibility rules or null for public routes
  final RouteAccessibility? accessibility;

  const AppRoute(this.name, {this.accessibility});

  RouteSettings settings({RouteArgs args = const {}}) => RouteSettings(name: name, arguments: args);

  Future<T> open(BuildContext context, {RouteArgs args = const {}});

  Future<T> openReplace(BuildContext context, {RouteArgs args = const {}, Object? result}) {
    Navigator.of(context).pop(result);
    return open(context, args: args);
  }

  Future<T> openForward<R>(BuildContext context, {RouteArgs args = const {}, ResultTransform<T, R>? transform}) {
    transform ??= (value) => value as R;
    final completer = Completer<R>();
    Navigator.of(context).pop(_ForwardResult<R>(completer.future));
    final raw = open(context, args: args);
    raw.then(transform).then(completer.complete, onError: completer.completeError);
    return raw;
  }

  static void declareRoutes(Set<ActivityRoute> routes) =>
      _routes.addEntries(routes.map((route) => MapEntry(route.name, route)));

  static Route onGenerateRoute(RouteSettings settings) {
    if (settings.arguments == null) settings = settings.copyWith(arguments: const <String, dynamic>{});
    final route = requireNotNull(_routes[settings.name], message: () => 'Route ${settings.name} not found');
    final accessibility = route.accessibility;
    if (accessibility != null && !accessibility.isAccessible(settings)) {
      final redirect = accessibility.redirect(settings);
      return onGenerateRoute(redirect);
    }
    return route.builder(settings);
  }

  static void redirect(BuildContext context, RouteSettings? settings) => settings == null
      ? Navigator.of(context).maybePop()
      : Navigator.of(context).pushReplacementNamed(settings.name!, arguments: settings.arguments);

  static Future<T> _result<T>(Future result) async =>
      result.then((value) async => value is _ForwardResult<T> ? await value.value : value as T);

  static String _routeName(Object data) => data is String ? data : data.toString().snakeCase;
}

class _ForwardResult<R> {
  final Future<R> value;

  const _ForwardResult(this.value);
}
